import {
  Carousel,
  CarouselContent,
  CarouselItem,
  type CarouselApi,
} from "@/components/ui/carousel";
import { Button } from "@/components/ui/button";
import { Link } from "wouter";
import { useEffect, useState, useCallback, useMemo, memo, useRef } from "react";
import Autoplay from 'embla-carousel-autoplay';
import { CAROUSEL_SLIDES, DEFAULT_FALLBACK_IMAGE } from "@/lib/carousel-config";
import type { EmblaPluginType } from 'embla-carousel';
import { preloadImages } from "../../lib/image";

// Memoized carousel item component to prevent unnecessary re-renders
const CarouselSlide = memo(({ 
  slide, 
  index, 
  isActive,
  onImageLoad,
  onImageError,
  loadedImages,
  imageErrors
}: {
  slide: typeof CAROUSEL_SLIDES[0],
  index: number,
  isActive: boolean,
  onImageLoad: (index: number) => void,
  onImageError: (index: number, isWebP: boolean) => void,
  loadedImages: Record<number, boolean>,
  imageErrors: Record<number, boolean>
}) => {
  // Use refs to minimize rendering and state updates
  const loaded = loadedImages[index] || false;
  const hasError = imageErrors[index] || false;
  
  // Skip rendering non-active slides for better performance
  // Only render the currently active slide and its neighbors
  if (!isActive && index !== ((currentIndex + 1) % CAROUSEL_SLIDES.length) && index !== ((currentIndex - 1 + CAROUSEL_SLIDES.length) % CAROUSEL_SLIDES.length)) {
    return (
      <CarouselItem 
        key={index} 
        className="relative"
        style={{ height: "min(100vh, 80vw)" }}
      />
    );
  }
  
  return (
    <CarouselItem 
      key={index} 
      className="relative"
      style={{ height: "min(100vh, 80vw)" }}
    >
      {/* Pre-allocate space with correct dimensions */}
      <div 
        className="absolute inset-0 bg-gray-100" 
        style={{ aspectRatio: "16/9", width: "100%", height: "100%" }}
      >
        {/* Only use picture tag for the active slide to reduce DOM complexity */}
        {isActive ? (
          <picture>
            <source
              srcSet={slide.image}
              type="image/webp"
              onError={() => onImageError(index, true)}
            />
            <img
              src={hasError ? DEFAULT_FALLBACK_IMAGE : slide.fallbackImage}
              alt={slide.headline}
              width="1920" 
              height="1080"
              className={`w-full h-full object-cover transition-opacity duration-300 ${
                loaded ? 'opacity-100' : 'opacity-0'
              }`}
              loading={index === 0 ? "eager" : "lazy"}
              onLoad={() => onImageLoad(index)}
              onError={() => onImageError(index, false)}
              decoding={index === 0 ? "sync" : "async"}
              // Add as data attribute to avoid React TypeScript errors
              data-fetchpriority={index === 0 ? "high" : "auto"}
            />
          </picture>
        ) : (
          // Simpler image tag for non-active slides
          <img
            src={hasError ? DEFAULT_FALLBACK_IMAGE : slide.image}
            alt={slide.headline}
            width="1920" 
            height="1080"
            className={`w-full h-full object-cover ${loaded ? 'opacity-100' : 'opacity-0'}`}
            loading="lazy"
            onLoad={() => onImageLoad(index)}
            onError={() => onImageError(index, false)}
          />
        )}
        <div 
          className="absolute inset-0 bg-gradient-to-t from-black/40 via-black/10 to-transparent"
          aria-hidden="true"
        />
      </div>

      {/* Only render content for the active slide to reduce TBT */}
      {isActive && (
        <div className="absolute inset-0 flex items-center">
          <div className="w-full max-w-7xl mx-auto px-4 sm:px-6 lg:px-8">
            <div className="max-w-3xl">
              <div className="backdrop-blur-md bg-white/40 p-8 rounded-xl border border-white/30 shadow-xl">
                <h1 className="text-xl sm:text-2xl md:text-3xl lg:text-4xl xl:text-5xl font-bold text-gray-900">
                  {slide.headline}
                </h1>
                <p className="text-sm sm:text-base md:text-lg lg:text-xl text-gray-800 mt-4">
                  {slide.description}
                </p>
                <div className="flex flex-col sm:flex-row gap-3 sm:gap-4 mt-8">
                  <Button 
                    size="lg"
                    className="w-full sm:w-auto bg-primary/90 text-white hover:bg-primary shadow-lg" 
                    asChild
                  >
                    <Link href={slide.primaryButtonHref}>
                      {slide.primaryButtonText}
                    </Link>
                  </Button>
                  <Button 
                    size="lg"
                    variant="outline" 
                    className="w-full sm:w-auto border-primary/80 text-primary hover:bg-primary/5 shadow-lg"
                    asChild
                  >
                    <Link href={slide.secondaryButtonHref}>
                      {slide.secondaryButtonText}
                    </Link>
                  </Button>
                </div>
              </div>
            </div>
          </div>
        </div>
      )}
    </CarouselItem>
  );
});

CarouselSlide.displayName = "CarouselSlide";

// Store current index in module scope to avoid prop drilling
let currentIndex = 0;

export default function CarouselHero() {
  const [api, setApi] = useState<CarouselApi>();
  const [slideIndex, setSlideIndex] = useState(0);
  const [loadedImages, setLoadedImages] = useState<Record<number, boolean>>({0: false});
  const [imageErrors, setImageErrors] = useState<Record<number, boolean>>({});
  const pluginRef = useRef<EmblaPluginType>();
  
  // Optimized image loading strategy to minimize TBT
  useEffect(() => {
    // Set initial index
    currentIndex = 0;
    
    // Implement progressive loading strategy with requestIdleCallback
    if ('requestIdleCallback' in window) {
      // First priority: Load only the first image using high priority
      const loadFirstImage = () => {
        const img = new Image();
        img.fetchPriority = 'high';
        img.decoding = 'sync';
        img.onload = () => setLoadedImages(prev => ({...prev, 0: true}));
        img.src = CAROUSEL_SLIDES[0].image;
      };
      
      // Second priority: Load the second slide during idle time
      const loadSecondImage = () => {
        if (CAROUSEL_SLIDES.length > 1) {
          (window as any).requestIdleCallback(() => {
            const img = new Image();
            img.fetchPriority = 'low';
            img.decoding = 'async';
            img.src = CAROUSEL_SLIDES[1].image;
          }, { timeout: 2000 });
        }
      };
      
      // Load the first image immediately
      loadFirstImage();
      
      // Defer loading the second image to avoid main thread blocking
      setTimeout(loadSecondImage, 300);
      
      // Preload remaining images only after interaction or significant delay
      setTimeout(() => {
        (window as any).requestIdleCallback(() => {
          // Use our optimized preloader that won't block the main thread
          const remainingImages = CAROUSEL_SLIDES.slice(2).map(slide => slide.image);
          preloadImages(remainingImages, 1, 'low');
        }, { timeout: 4000 });
      }, 2000);
    } else {
      // Fallback for browsers without requestIdleCallback
      // Load first image immediately
      const img = new Image();
      img.onload = () => setLoadedImages(prev => ({...prev, 0: true}));
      img.src = CAROUSEL_SLIDES[0].image;
      
      // Stagger remaining loads with increasing delays
      CAROUSEL_SLIDES.slice(1).forEach((slide, idx) => {
        setTimeout(() => {
          const img = new Image();
          img.src = slide.image;
        }, 1000 + idx * 500);
      });
    }
  }, []);

  // Create the autoplay plugin only once and with simpler settings
  if (!pluginRef.current) {
    pluginRef.current = Autoplay({
      delay: 7000,
      stopOnInteraction: true // Stop on interaction to reduce CPU usage
    }) as unknown as EmblaPluginType;
  }

  // Memoized carousel options with simpler settings
  const carouselOptions = useMemo(() => ({
    loop: true,
    align: "start" as const,
    skipSnaps: false,
    watchDrag: false,
    dragFree: false // Reduces computational overhead
  }), []);

  // Batch state updates for better performance
  const handleImageError = useCallback((index: number, isWebP: boolean) => {
    if (!isWebP) {
      setImageErrors(prev => ({...prev, [index]: true}));
    }
  }, []);

  const handleImageLoad = useCallback((index: number) => {
    // Don't update state if already loaded (prevents unnecessary rerenders)
    if (!loadedImages[index]) {
      setLoadedImages(prev => ({...prev, [index]: true}));
    }
  }, [loadedImages]);

  // Handle slide changes more efficiently
  useEffect(() => {
    if (!api) return;

    const handleSelect = () => {
      const newIndex = api.selectedScrollSnap();
      setSlideIndex(newIndex);
      currentIndex = newIndex;
      
      // Preload the next image in the carousel with a slight delay
      // This ensures we're not loading all images at once causing TBT
      const nextIndex = (newIndex + 1) % CAROUSEL_SLIDES.length;
      if (!loadedImages[nextIndex]) {
        setTimeout(() => {
          const img = new Image();
          img.src = CAROUSEL_SLIDES[nextIndex].image;
        }, 300);
      }
    };

    api.on("select", handleSelect);
    handleSelect(); // Initial update
    
    // Cleanup function must return void
    return () => {
      api.off("select", handleSelect);
    };
  }, [api, loadedImages]);

  return (
    <section className="relative w-full min-h-screen">
      <Carousel
        opts={carouselOptions}
        plugins={[pluginRef.current] as any}
        setApi={setApi}
        className="relative w-full"
        style={{ height: "min(100vh, 80vw)" }}
      >
        <CarouselContent>
          {CAROUSEL_SLIDES.map((slide, index) => (
            <CarouselSlide
              key={index}
              slide={slide}
              index={index}
              isActive={index === slideIndex}
              onImageLoad={handleImageLoad}
              onImageError={handleImageError}
              loadedImages={loadedImages}
              imageErrors={imageErrors}
            />
          ))}
        </CarouselContent>
      </Carousel>
    </section>
  );
}